<?php
// ============================================================================
// GD-CMS: A heirarchical content management system
// Version 1.2
// ============================================================================
//
// Copyright (c) 2010-2011 General Development
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// ============================================================================
// 
// requires at least PHP verion 4.02 (for the __FILE__ constant)
// Tested with PHP 5.2.10.



// Parameters:
// $_GET['l1'], $_GET['l2'],...etc. The sections of each level of the menu heirarchy tree to descend into until a file (not directory) is encountered.

$files_path = dirname(__FILE__) . '/';
if(!isset($use_url_rewriting)) $use_url_rewriting = FALSE;
if(!isset($href_prefix)) $href_prefix = '';
if(!isset($content_path)) $content_path = $files_path .'content/';
if(!isset($gdcms_double_encode)) $gdcms_double_encode = FALSE;

$found = FALSE;
//populate an array of sections for each heirarchy level until we get down to a leaf node.
// (files are leaf nodes, directories are not)
for($level=1; ; $level++)
{
    $numhidden = 0;

    $dirlisting[$level] = scandir($content_path);

    //Index the directory listing for (each) current level into titles and filenames by section index number
    //$fileindex[level][section] and $titleindex[level][section].
    foreach($dirlisting[$level] as $filename) {
        $delimpos = strpos($filename, '.');
        $index = substr($filename, 0, $delimpos);
        $title = substr($filename, $delimpos+1);
        if(!is_dir($content_path . $filename)) { //non-directories have a filename extension (exclude it)
            $delimpos = strrpos($title, '.');
            if($delimpos !== FALSE)
                $title = substr($title, 0, $delimpos);
        }
        
        if($index == 'h') { //ensure all hidden items get a unique index
            $numhidden++;
            $index .= $numhidden;
        }

        $fileindex[$level][$index] = $filename;
        $titleindex[$level][$index] = $title;
        
    }

    if(isset($_GET['l'.$level])) {
        $lastlevelspecified = TRUE;
        foreach($titleindex[$level] as $index=>$thistitle) 
            if(strcasecmp($thistitle, stripslashes($_GET['l'.$level]))==0) {
                $indexlevel[$level] = $index;
                break;
            }
    }
    else { //default to 'd.xxxx...' item, or failing that, item 1 at this level
        $lastlevelspecified = FALSE;
        if (isset($fileindex[$level]['d'])) $indexlevel[$level] = 'd';
        else if (isset($fileindex[$level][1])) {
            $indexlevel[$level] = 1;
            $lastlevelspecified = TRUE;
        }
    }

    if(isset($indexlevel[$level])) { //$fileindex[$level][$indexlevel[$level]])) {
        $filename = $fileindex[$level][$indexlevel[$level]];
        $content_path .= $filename;
        if(is_dir($content_path))
            $content_path .= '/'; //add trailing slash to directory
        else { //we have a leaf node!
            $found = TRUE;
            break;
        }
    }
    else {
        $found = FALSE;
        if($lastlevelspecified)
            header('HTTP/1.0 404 Not Found');
        else
            header('HTTP/1.1 403 Forbidden');
        break;
    }
}
$depth = $level;


//gets the title correspoinding to particular section# in a particular menu level#
//along the branch of the current document
function gettitle($level, $section) {
    global $titleindex;
    if(isset($titleindex[$level][$section]))
        return $titleindex[$level][$section];
    else
        return '[Not found]';
}


function printtitle() {
    global $depth;
    global $indexlevel;

    echo gettitle($depth, $indexlevel[$depth]);
}


function printbody() {
    global $found, $lastlevelspecified, $content_path;

    if($found) //is it a leaf node?
        include($content_path);
    else if($lastlevelspecified)
        echo '<h1>Error<sub>404</sub></h1>The location wasn\'t found.';
    else
        echo '<h1>Error<sub>403</sub></h1>There is no content to browse at this level.';
}

function _gdcms_encode($url) {
    global $gdcms_double_encode;
    if($gdcms_double_encode)
        return urlencode(urlencode($url));
    return urlencode($url);
}

function printmenu($level) {
    global $depth;
    global $dirlisting;
    global $fileindex;
    global $titleindex;
    global $indexlevel;
    global $lastlevelspecified;
    global $use_url_rewriting;
    global $href_prefix;

    for ($i=1; ; $i++) { //print each link in order
        if (isset($fileindex[$level][$i])) { //print the i'th link
            $isactiveitem = ($i == $indexlevel[$level]);
            echo "<span id=\"menul{$level}s{$i}\" class=\"menuitem";
            if($isactiveitem) echo ' menuactiveitem';
            echo "\">";
            if(!$isactiveitem || ($level != $depth && !($level+1==$depth && !$lastlevelspecified))) {
                echo '<a href="' . $href_prefix;
                echo (!$use_url_rewriting) ? '?' : '/';
                for ($j=1; $j<$level; $j++) {
                    echo !$use_url_rewriting ? "l$j=" : '';
                    echo _gdcms_encode($titleindex[$j][$indexlevel[$j]]);
                    echo $use_url_rewriting ? '/' : '&';
                }
                echo !$use_url_rewriting ? "l$level=" : '';
                echo _gdcms_encode($titleindex[$level][$i]);
                echo '">';
                echo gettitle($level, $i);
                echo '</a>';
            }
            else {
                echo gettitle($level, $i);
            }
            echo '</span> ';
        }
        else { //we've reached the end of the menu list.
            break;
        }
    }
}

function gdcms_place_promo() {
    echo 'Powered by GD-CMS version 1.';
}

?>
